@@ -357,6 +357,9 @@ class InfrastructureInfo:
 
         with self._lock:
             original_radl = self.radl.clone()
+            # Add the description field
+            if radl.description:
+                self.radl.add(radl.description.clone(), "replace")
             # Add new networks ad ansible_hosts only
             for s in radl.networks + radl.ansible_hosts:
                 if not self.radl.add(s.clone(), "ignore") and warn:
@@ -449,17 +452,17 @@ class InfrastructureInfo:
         # TODO: Do in a better way
         radl = self.radl.clone()
         deploys = []
-        for deploy in radl.deploys:
-            if not deploy.id.startswith(self.FAKE_SYSTEM):
-                deploys.append(deploy)
+        for dep in radl.deploys:
+            if not dep.id.startswith(self.FAKE_SYSTEM):
+                deploys.append(dep)
         radl.deploys = deploys
 
         # remove the F0000__FAKE_SYSTEM__ deploys
         # TODO: Do in a better way
         systems = []
-        for system in radl.systems:
-            if not system.name.startswith(self.FAKE_SYSTEM):
-                systems.append(system)
+        for syst in radl.systems:
+            if not syst.name.startswith(self.FAKE_SYSTEM):
+                systems.append(syst)
         radl.systems = systems
 
         return radl
@@ -686,15 +689,6 @@ class InfrastructureInfo:
         res = False
         for other_im_auth in auth.getAuthInfo("InfrastructureManager"):
             res = True
-            for elem in ['username', 'password']:
-                if elem not in other_im_auth:
-                    res = False
-                    break
-                if elem not in self_im_auth:
-                    InfrastructureInfo.logger.error("Inf ID %s has not elem %s in the auth data" % (self.id, elem))
-                if self_im_auth[elem] != other_im_auth[elem]:
-                    res = False
-                    break
 
             if 'token' in self_im_auth:
                 if 'token' not in other_im_auth:
@@ -710,8 +704,23 @@ class InfrastructureInfo:
                     res = False
                     break
 
+                # Username must start with the defined prefix
+                if not other_im_auth['username'].startswith(InfrastructureInfo.OPENID_USER_PREFIX):
+                    res = False
+                    break
+
                 # In case of OIDC token update it in each call to get a fresh version
                 self_im_auth['token'] = other_im_auth['token']
+            else:
+                for elem in ['username', 'password']:
+                    if elem not in other_im_auth:
+                        res = False
+                        break
+                    if elem not in self_im_auth:
+                        InfrastructureInfo.logger.error("Inf ID %s has not elem %s in the auth data" % (self.id, elem))
+                    if self_im_auth[elem] != other_im_auth[elem]:
+                        res = False
+                        break
 
             if (self_im_auth['username'].startswith(InfrastructureInfo.OPENID_USER_PREFIX) and
                     'token' not in other_im_auth):
@@ -731,6 +740,11 @@ class InfrastructureInfo:
             for self_im_auth in self.auth.getAuthInfo("InfrastructureManager"):
                 if self._is_authorized(self_im_auth, auth):
                     return True
+                if Config.ADMIN_USER:
+                    admin_auth = dict(Config.ADMIN_USER)
+                    admin_auth["type"] = "InfrastructureManager"
+                    if self._is_authorized(admin_auth, auth):
+                        return True
 
             return False
         else:
