@@ -28,6 +28,7 @@ import IM.InfrastructureList
 
 from IM.VMRC import VMRC
 from IM.AppDBIS import AppDBIS
+from IM.AppDB import AppDB
 from IM.CloudInfo import CloudInfo
 from IM.auth import Authentication
 from IM.recipe import Recipe
@@ -35,7 +36,7 @@ from IM.config import Config
 from IM.VirtualMachine import VirtualMachine
 
 from radl import radl_parse
-from radl.radl import Feature, RADL
+from radl.radl import Feature, RADL, system
 from radl.radl_json import dump_radl as dump_radl_json
 
 from IM.openid.JWT import JWT
@@ -386,6 +387,26 @@ class InfrastructureManager:
 
         return concrete_system, score
 
+    @staticmethod
+    def search_vm(inf, radl_sys, auth):
+        # If an images is already set do not search
+        if radl_sys.getValue("disk.0.image.url"):
+            return []
+
+        dist = radl_sys.getValue('disk.0.os.flavour')
+        version = radl_sys.getValue('disk.0.os.version')
+        res = []
+        for c in CloudInfo.get_cloud_list(auth):
+            cloud_site = c.getCloudConnector(inf)
+            for image in cloud_site.list_images(auth):
+                if ((dist is None or dist.lower() in image["name"].lower()) and
+                        (version is None or version.lower() in image["name"].lower())):
+                    new_sys = system(radl_sys.name)
+                    new_sys.setValue("disk.0.image.url", image["uri"])
+                    res.append(new_sys)
+                    break
+        return res
+
     @staticmethod
     def systems_with_iis(sel_inf, radl, auth):
         """
@@ -411,9 +432,6 @@ class InfrastructureManager:
         for system_id in set([d.id for d in radl.deploys if d.vm_number > 0]):
             s = radl.get_system_by_name(system_id)
 
-            if not s.getValue("disk.0.image.url") and len(vmrc_list + appdbis_list) == 0:
-                raise Exception("No correct VMRC or AppDBIS auth data provided nor image URL")
-
             if Config.SINGLE_SITE:
                 image_id = os.path.basename(s.getValue("disk.0.image.url"))
                 url_prefix = Config.SINGLE_SITE_IMAGE_URL_PREFIX
@@ -435,13 +453,14 @@ class InfrastructureManager:
 
             vmrc_res = [s0 for vmrc in vmrc_list for s0 in vmrc.search_vm(s)]
             appdbis_res = [s0 for appdbis in appdbis_list for s0 in appdbis.search_vm(s)]
+            local_res = InfrastructureManager.search_vm(sel_inf, s, auth)
             # Check that now the image URL is in the RADL
-            if not s.getValue("disk.0.image.url") and not vmrc_res and not appdbis_res:
-                sel_inf.add_cont_msg("No VMI obtained from VMRC nor AppDBIS to system: " + system_id)
-                raise Exception("No VMI obtained from VMRC nor AppDBIS to system: " + system_id)
+            if not s.getValue("disk.0.image.url") and not vmrc_res and not appdbis_res and not local_res:
+                sel_inf.add_cont_msg("No VMI obtained from VMRC nor AppDBIS nor Sites to system: " + system_id)
+                raise Exception("No VMI obtained from VMRC nor AppDBIS not Sites to system: " + system_id)
 
             n = [s_without_apps.clone().applyFeatures(s0, conflict="other", missing="other")
-                 for s0 in (vmrc_res + appdbis_res)]
+                 for s0 in (vmrc_res + appdbis_res + local_res)]
             systems_with_vmrc[system_id] = n if n else [s_without_apps]
 
         return systems_with_vmrc
@@ -1446,6 +1465,40 @@ class InfrastructureManager:
                 InfrastructureManager.logger.error("Error getting auth data from AppDBIS: %s" % sites)
         return auth
 
+    @staticmethod
+    def translate_egi_to_ost(auth):
+        # Gen OST auth for all EGI auth sent
+        res = []
+        for auth_item in auth.auth_list:
+            if auth_item.get('type') == "EGI":
+                if 'host' in auth_item and 'vo' in auth_item and 'token' in auth_item:
+                    ost_auth = {'id': auth_item['id'], 'type': 'OpenStack', 'username': 'egi.eu', 'tenant': 'openid',
+                                'password': auth_item['token'], 'auth_version': '3.x_oidc_access_token',
+                                'vo': auth_item['vo']}
+                    site_id = AppDB.get_site_id(auth_item["host"], stype="openstack")
+                    site_url = AppDB.get_site_url(site_id)
+                    if not site_url:
+                        InfrastructureManager.logger.error("Site name '%s' not found at AppDB." % auth_item['host'])
+                        continue
+                    ost_auth['host'] = site_url
+                    projects = AppDB.get_project_ids(site_id)
+                    # If the VO does not appear in the project IDs
+                    if auth_item['vo'] in projects:
+                        ost_auth['domain'] = projects[auth_item['vo']]
+                    else:
+                        # let's use the VO name directly
+                        ost_auth['domain'] = auth_item['vo']
+
+                    if 'api_version' in auth_item:
+                        ost_auth['api_version'] = auth_item['api_version']
+
+                    res.append(ost_auth)
+            else:
+                res.append(auth_item)
+
+        auth.auth_list = res
+        return auth
+
     @staticmethod
     def check_auth_data(auth):
         # First check if it is configured to check the users from a list
@@ -1487,6 +1540,7 @@ class InfrastructureManager:
         # We have to check if TTS is needed for other auth item
         auth = InfrastructureManager.get_auth_from_vault(auth)
         auth = InfrastructureManager.gen_auth_from_appdb(auth)
+        auth = InfrastructureManager.translate_egi_to_ost(auth)
         return auth
 
     @staticmethod
@@ -1780,3 +1834,27 @@ class InfrastructureManager:
             sel_inf.change_auth(new_auth, overwrite)
         IM.InfrastructureList.InfrastructureList.save_data(inf_id)
         return ""
+
+    @staticmethod
+    def GetInfrastructureOwners(inf_id, auth):
+        """
+        Get the list of owners of an infrastructure.
+
+        Args:
+
+        - inf_id(str): infrastructure id.
+        - auth(Authentication): parsed authentication tokens.
+
+        Return: list of str
+        """
+        auth = InfrastructureManager.check_auth_data(auth)
+
+        InfrastructureManager.logger.info("Getting RADL of the Inf ID: " + str(inf_id))
+
+        sel_inf = InfrastructureManager.get_infrastructure(inf_id, auth)
+
+        res = []
+        for im_auth in sel_inf.auth.getAuthInfo("InfrastructureManager"):
+            res.append(im_auth['username'])
+
+        return res
