@@ -169,7 +169,7 @@ class AnsibleCallbacks(CallbackBase):
         else:
             msg += ": [%s]" % result._host.get_name()
 
-        msg += " => (item=%s)" % (self._get_item(result._result),)
+        msg += " => (item=%s)" % (self._get_item_label(result._result),)
 
         if ((self._display.verbosity > 0 or '_ansible_verbose_always' in result._result) and
                 '_ansible_verbose_override' not in result._result):
@@ -196,14 +196,14 @@ class AnsibleCallbacks(CallbackBase):
         else:
             msg += "[%s]" % (result._host.get_name())
 
-        self._display.display(msg + " (item=%s) => %s" % (self._get_item(result._result),
+        self._display.display(msg + " (item=%s) => %s" % (self._get_item_label(result._result),
                                                           self._dump_results(result._result)),
                               color=C.COLOR_ERROR)
         self._handle_warnings(result._result)
 
     def v2_runner_item_on_skipped(self, result):
         if C.DISPLAY_SKIPPED_HOSTS:
-            msg = "skipping: [%s] => (item=%s) " % (result._host.get_name(), self._get_item(result._result))
+            msg = "skipping: [%s] => (item=%s) " % (result._host.get_name(), self._get_item_label(result._result))
             if ((self._display.verbosity > 0 or '_ansible_verbose_always' in result._result) and
                     '_ansible_verbose_override' not in result._result):
                 msg += " => %s" % self._dump_results(result._result)
