@@ -14,8 +14,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import defusedxml.xmlrpc
-defusedxml.xmlrpc.monkey_patch()
+from defusedxml import xmlrpc
+xmlrpc.monkey_patch()
 
 try:
     from xmlrpclib import ServerProxy  # nosec
@@ -37,6 +37,7 @@ from IM.config import ConfigOpenNebula
 from netaddr import IPNetwork, IPAddress
 from IM.config import Config
 from IM.tts.onetts import ONETTSClient
+from IM.SSH import SSH
 
 # Set of classes to parse the XML results of the ONE API
 
@@ -392,6 +393,7 @@ class OpenNebulaCloudConnector(CloudConnector):
 
             # Update network data
             self.setIPsFromTemplate(vm, res_vm.TEMPLATE)
+            self.manage_dns_entries("add", vm, auth_data)
 
             # Update disks data
             self.setDisksFromTemplate(vm, res_vm.TEMPLATE)
@@ -577,6 +579,11 @@ class OpenNebulaCloudConnector(CloudConnector):
             err = ""
             success = True
 
+        try:
+            self.manage_dns_entries("del", vm, auth_data)
+        except NotImplementedError:
+            self.log_warn("Trying to delete DNS entries on ONE.")
+
         if last and success:
             one_ver = self.getONEVersion(auth_data)
             # Security Groups appears in version 4.12.0
@@ -693,7 +700,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         public = system.getValue('disk.0.os.credentials.public_key')
 
         if not password and (not private or not public):
-            (public, private) = self.keygen()
+            (public, private) = SSH.keygen()
             system.setValue('disk.0.os.credentials.private_key', private)
 
         if (private and public) or ConfigOpenNebula.TEMPLATE_CONTEXT or Config.SSH_REVERSE_TUNNELS:
