@@ -22,6 +22,7 @@ import tempfile
 import requests
 import base64
 from IM.CloudInfo import CloudInfo
+from IM.SSH import SSH
 
 try:
     from libcloud.common.exceptions import BaseHTTPError
@@ -363,6 +364,13 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             if not username:
                 res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
 
+            auths = auth_data.getAuthInfo(self.type, self.cloud.server)
+            if auths:
+                auth = self.get_auth(auths)
+                if auth.get("vo"):
+                    res_system.addFeature(Feature("provider.vo", "=", auth.get("vo")),
+                                          conflict="other", missing="other")
+
             return res_system
         else:
             return None
@@ -546,7 +554,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             self.addAdditionalIP(vm, node.driver)
             self.addRouterInstance(vm, node.driver)
             self.setIPsFromInstance(vm, node)
-            self.add_dns_entries(vm, auth_data)
+            self.manage_dns_entries("add", vm, auth_data)
             self.setVolumesInfo(vm, node)
         else:
             self.log_warn("Error updating the instance %s. VM not found." % vm.id)
@@ -554,37 +562,27 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return (True, vm)
 
-    def add_dns_entries(self, vm, auth_data):
-        """
-        Add the required entries in the AWS Route53 service
-
-        Arguments:
-           - vm(:py:class:`IM.VirtualMachine`): VM information.
-           - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
-        """
-        try:
-            dns_entries = self.get_dns_entries(vm)
-            if dns_entries:
-                for hostname, domain, ip in dns_entries:
-                    # Special case for EGI DyDNS
-                    # format of the hostname: dydns:secret@hostname
-                    if hostname.startswith("dydns:") and "@" in hostname:
-                        parts = hostname[6:].split("@")
-                        auth = "%s.%s:%s" % (parts[1], domain[:-1], parts[0])
-                        headers = {"Authorization": "Basic %s" % base64.b64encode(auth.encode()).decode()}
-                        url = "https://nsupdate.fedcloud.eu/nic/update?hostname=%s.%s&myip=%s" % (parts[1],
-                                                                                                  domain[:-1],
-                                                                                                  ip)
-                        resp = requests.get(url, headers=headers)
-                        resp.raise_for_status()
-                    else:
-                        # TODO: https://docs.openstack.org/designate/latest/index.html
-                        return False
-            return True
-        except Exception as ex:
-            self.error_messages += "Error creating DNS entries %s.\n" % str(ex)
-            self.log_exception("Error creating DNS entries")
-            return False
+    def add_dns_entry(self, hostname, domain, ip, auth_data, extra_args=None):
+        # Special case for EGI DyDNS
+        # format of the hostname: dydns:secret@hostname
+        if hostname.startswith("dydns:") and "@" in hostname:
+            parts = hostname[6:].split("@")
+            auth = "%s.%s:%s" % (parts[1], domain[:-1], parts[0])
+            headers = {"Authorization": "Basic %s" % base64.b64encode(auth.encode()).decode()}
+            url = "https://nsupdate.fedcloud.eu/nic/update?hostname=%s.%s&myip=%s" % (parts[1],
+                                                                                      domain[:-1],
+                                                                                      ip)
+            try:
+                resp = requests.get(url, headers=headers)
+                resp.raise_for_status()
+            except Exception as ex:
+                self.error_messages += "Error creating DNS entries %s.\n" % str(ex)
+                self.log_exception("Error creating DNS entries")
+                return False
+        else:
+            # TODO: https://docs.openstack.org/designate/latest/index.html
+            raise NotImplementedError("Should have implemented this")
+        return True
 
     @staticmethod
     def map_radl_ost_networks(vm, ost_nets):
@@ -887,7 +885,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return net_map
 
-    def get_router_public(self, driver, radl):
+    def get_router_public(self, driver, radl, inf_id, create=False):
         try:
             # Get the public net provider id
             pub_net_provider_id = None
@@ -920,11 +918,20 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     return routers[list(routers.keys())[0]]
                 else:
                     self.log_warn("No public router found!.")
-                    return None
 
         except Exception:
             self.log_exception("Error getting public router.")
 
+        # try to create a router
+        if create and pub_nets:
+            try:
+                gateway_info = {'network_id': list(pub_nets.keys())[0]}
+                name = "im-%s" % (inf_id)
+                return driver.ex_create_router(name, description="IM created router",
+                                               external_gateway_info=gateway_info)
+            except Exception:
+                self.log_exception("Error creating public router.")
+
         return None
 
     @staticmethod
@@ -941,7 +948,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         """
         Delete created OST networks
         """
-        router = self.get_router_public(driver, inf.radl)
+        router = self.get_router_public(driver, inf.radl, inf.id, create=False)
         msg = ""
         res = True
         for ost_net in driver.ex_list_networks():
@@ -967,6 +974,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     self.log_info("Deleting net %s." % ost_net.name)
                     driver.ex_delete_network(ost_net)
 
+        if router and router.name == "im-%s" % inf.id:
+            driver.ex_delete_router(router.name)
+
         return res, msg
 
     def create_networks(self, driver, radl, inf):
@@ -975,7 +985,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         """
         try:
             i = 0
-            router = self.get_router_public(driver, radl)
+            router = self.get_router_public(driver, radl, inf.id, create=True)
 
             while radl.systems[0].getValue("net_interface." + str(i) + ".connection"):
                 net_name = radl.systems[0].getValue("net_interface." + str(i) + ".connection")
@@ -1055,6 +1065,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 self.delete_networks(driver, inf)
             except Exception:
                 self.log_exception("Error deleting networks.")
+
             raise Exception("Error creating networks: %s" % ext.args[0])
 
         return True
@@ -1280,7 +1291,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         public_key = system.getValue("disk.0.os.credentials.public_key")
         if not public_key:
             # We must generate them
-            (public_key, private_key) = self.keygen()
+            (public_key, private_key) = SSH.keygen()
             system.setValue('disk.0.os.credentials.private_key', private_key)
 
         if "ssh_key" in driver.features.get("create_node", []):
@@ -1341,6 +1352,11 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     driver.ex_delete_security_group(sg)
                 except Exception as ex:
                     self.log_exception("Error deleting security group: %s." % get_ex_error(ex))
+                self.log_info("Deleting networks.")
+                try:
+                    self.delete_networks(driver, inf)
+                except Exception:
+                    self.log_exception("Error deleting networks.")
 
         return res
 
@@ -1609,7 +1625,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     self.log_debug("Dettaching volume %s." % vol_id)
                     volume = node.driver.ex_get_volume(vol_id)
                     node.driver.detach_volume(volume)
-                except Exception as ex:
+                except Exception:
                     self.log_exception("Error dettaching volume %s." % vol_id)
 
             for sg_name in self._get_security_names(vm.inf):
@@ -1617,7 +1633,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     self.log_debug("Dettaching SG %s." % sg_name)
                     security_group = OpenStackSecurityGroup(None, None, sg_name, "", node.driver)
                     node.driver.ex_remove_security_group_from_node(security_group, node)
-                except Exception as ex:
+                except Exception:
                     self.log_exception("Error dettaching SG %s." % sg_name)
 
             res = node.destroy()
@@ -1651,6 +1667,21 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         else:
             self.log_warn("VM " + str(vm.id) + " not found.")
 
+        try:
+            res = self.manage_dns_entries("del", vm, auth_data)
+            if not res:
+                msg = "Error deleting DNS entries."
+            else:
+                msg = ""
+        except NotImplementedError:
+            self.log_warn("Trying to delete DNS entries on OST.")
+        except Exception as ex:
+            res = False
+            msg = get_ex_error(ex)
+            msgs.append(msg)
+        success.append(res)
+        msgs.append(msg)
+
         driver = self.get_driver(auth_data)
 
         if last:
@@ -1999,7 +2030,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         return images
 
     @staticmethod
-    def _get_tenant_id(driver, auth):
+    def _get_tenant_id(auth):
         """
         Workaround function to get tenant id from tenant name
         """
@@ -2015,7 +2046,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
     def get_quotas(self, auth_data):
         driver = self.get_driver(auth_data)
-        tenant_id = self._get_tenant_id(driver, auth_data.getAuthInfo(self.type, self.cloud.server)[0])
+        tenant_id = self._get_tenant_id(auth_data.getAuthInfo(self.type, self.cloud.server)[0])
         quotas = driver.ex_get_quota_set(tenant_id)
         try:
             net_quotas = driver.ex_get_network_quotas(tenant_id)
